<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Context;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Exception\QuoteNotFoundException;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Channel\Context\ChannelNotFoundException;
use Sylius\Component\Customer\Context\CustomerContextInterface;

class CustomerAndChannelBasedQuoteContext implements QuoteContextInterface
{
    private ChannelContextInterface $channelContext;

    private CustomerContextInterface $customerContext;

    private QuoteRepositoryInterface $quoteRepository;

    public function __construct(
        ChannelContextInterface $channelContext,
        CustomerContextInterface $customerContext,
        QuoteRepositoryInterface $quoteRepository
    ) {
        $this->channelContext = $channelContext;
        $this->customerContext = $customerContext;
        $this->quoteRepository = $quoteRepository;
    }

    public function getQuote(): QuoteInterface
    {
        try {
            $channel = $this->channelContext->getChannel();
        } catch (ChannelNotFoundException $exception) {
            throw new QuoteNotFoundException('Sylius was not able to find the quote, as there is no current channel.');
        }

        $customer = $this->customerContext->getCustomer();
        if (null === $customer) {
            throw new QuoteNotFoundException('Sylius was not able to find the quote, as there is no logged in user.');
        }

        $quote = $this->quoteRepository->findLatestQuoteByChannelAndCustomer($channel, $customer);
        if (null === $quote) {
            throw new QuoteNotFoundException('Sylius was not able to find the quote for currently logged in user.');
        }

        return $quote;
    }
}
