<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\DependencyInjection;

use Dedi\SyliusQuotePlugin\Controller\QuoteAuthorizationChecker;
use Dedi\SyliusQuotePlugin\EventListener\Sales\OrderItemPreDeleteListener;
use Sylius\Bundle\ResourceBundle\DependencyInjection\Extension\AbstractResourceExtension;
use Symfony\Component\Config\Definition\ConfigurationInterface;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;
use Symfony\Component\DependencyInjection\Reference;

final class DediSyliusQuoteExtension extends AbstractResourceExtension
{
    public function load(array $configs, ContainerBuilder $container): void
    {
        $config = $this->processConfiguration($this->getConfiguration([], $container), $configs);
        $loader = new XmlFileLoader($container, new FileLocator(__DIR__ . '/../Resources/config'));
        $this->registerResources('dedi_quote', $config['driver'], $config['resources'], $container);

        $loader->load('services.xml');

        if (!$config['allow_anonymous_quote']) {
            $this->registerAuthorizationChecker($container);
        }

        if (!$config['allow_cart_edit']) {
            $this->registerCartEvents($container);
        }
    }

    public function getConfiguration(array $config, ContainerBuilder $container): ConfigurationInterface
    {
        return new Configuration();
    }

    private function registerAuthorizationChecker(ContainerBuilder $container): void
    {
        $container
            ->register('dedi_quote.resource_controller.authorization_checker', QuoteAuthorizationChecker::class)
            ->addArgument(new Reference('security.helper'))
            ->addArgument(new Reference('dedi_quote.resource_controller.authorization_checker.inner'))
            ->addArgument(new Reference('sylius.context.customer'))
            ->setDecoratedService('sylius.resource_controller.authorization_checker.disabled')
        ;
    }

    private function registerCartEvents(ContainerBuilder $container): void
    {
        $container
            ->register('dedi_quote.event_listener.order_item_pre_delete', OrderItemPreDeleteListener::class)
            ->addArgument(new Reference('dedi_quote.specification.is_order_item_from_quote'))
            ->addTag('kernel.event_listener', ['event' => 'sylius.order_item.pre_remove']);
    }
}
